From d2ecba21b3c7e45fdd8e962853ba0a5cc142a316 Mon Sep 17 00:00:00 2001
From: Amul Sul <amul.sul@enterprisedb.com>
Date: Wed, 14 Dec 2022 12:22:36 +0530
Subject: [PATCH v1 23/25] Change tidin to allow non-throw error reporting

---
 src/backend/utils/adt/tid.c       |  9 +++++----
 src/test/regress/expected/tid.out | 25 +++++++++++++++++++++++++
 src/test/regress/sql/tid.sql      |  6 ++++++
 3 files changed, 36 insertions(+), 4 deletions(-)

diff --git a/src/backend/utils/adt/tid.c b/src/backend/utils/adt/tid.c
index 83ac589f957..4dc1b327bf3 100644
--- a/src/backend/utils/adt/tid.c
+++ b/src/backend/utils/adt/tid.c
@@ -57,6 +57,7 @@ Datum
 tidin(PG_FUNCTION_ARGS)
 {
 	char	   *str = PG_GETARG_CSTRING(0);
+	Node	   *escontext = fcinfo->context;
 	char	   *p,
 			   *coord[NTIDARGS];
 	int			i;
@@ -71,7 +72,7 @@ tidin(PG_FUNCTION_ARGS)
 			coord[i++] = p + 1;
 
 	if (i < NTIDARGS)
-		ereport(ERROR,
+		ereturn(escontext, (Datum) 0,
 				(errcode(ERRCODE_INVALID_TEXT_REPRESENTATION),
 				 errmsg("invalid input syntax for type %s: \"%s\"",
 						"tid", str)));
@@ -79,7 +80,7 @@ tidin(PG_FUNCTION_ARGS)
 	errno = 0;
 	cvt = strtoul(coord[0], &badp, 10);
 	if (errno || *badp != DELIM)
-		ereport(ERROR,
+		ereturn(escontext, (Datum) 0,
 				(errcode(ERRCODE_INVALID_TEXT_REPRESENTATION),
 				 errmsg("invalid input syntax for type %s: \"%s\"",
 						"tid", str)));
@@ -93,7 +94,7 @@ tidin(PG_FUNCTION_ARGS)
 #if SIZEOF_LONG > 4
 	if (cvt != (unsigned long) blockNumber &&
 		cvt != (unsigned long) ((int32) blockNumber))
-		ereport(ERROR,
+		ereturn(escontext, (Datum) 0,
 				(errcode(ERRCODE_INVALID_TEXT_REPRESENTATION),
 				 errmsg("invalid input syntax for type %s: \"%s\"",
 						"tid", str)));
@@ -102,7 +103,7 @@ tidin(PG_FUNCTION_ARGS)
 	cvt = strtoul(coord[1], &badp, 10);
 	if (errno || *badp != RDELIM ||
 		cvt > USHRT_MAX)
-		ereport(ERROR,
+		ereturn(escontext, (Datum) 0,
 				(errcode(ERRCODE_INVALID_TEXT_REPRESENTATION),
 				 errmsg("invalid input syntax for type %s: \"%s\"",
 						"tid", str)));
diff --git a/src/test/regress/expected/tid.out b/src/test/regress/expected/tid.out
index 8cd6d605952..ee928d3b0a8 100644
--- a/src/test/regress/expected/tid.out
+++ b/src/test/regress/expected/tid.out
@@ -94,3 +94,28 @@ SELECT currtid2('tid_view_fake_ctid'::text, '(0,1)'::tid); -- fails
 ERROR:  ctid isn't of type TID
 DROP VIEW tid_view_fake_ctid;
 DROP TABLE tid_tab CASCADE;
+-- test non-error-throwing API for some core types
+SELECT pg_input_is_valid('(0)', 'tid');
+ pg_input_is_valid 
+-------------------
+ f
+(1 row)
+
+SELECT pg_input_error_message('(0)', 'tid');
+          pg_input_error_message          
+------------------------------------------
+ invalid input syntax for type tid: "(0)"
+(1 row)
+
+SELECT pg_input_is_valid('(0,-1)', 'tid');
+ pg_input_is_valid 
+-------------------
+ f
+(1 row)
+
+SELECT pg_input_error_message('(0,-1)', 'tid');
+           pg_input_error_message            
+---------------------------------------------
+ invalid input syntax for type tid: "(0,-1)"
+(1 row)
+
diff --git a/src/test/regress/sql/tid.sql b/src/test/regress/sql/tid.sql
index 990d314a5f8..5909f7b0b17 100644
--- a/src/test/regress/sql/tid.sql
+++ b/src/test/regress/sql/tid.sql
@@ -64,3 +64,9 @@ SELECT currtid2('tid_view_fake_ctid'::text, '(0,1)'::tid); -- fails
 DROP VIEW tid_view_fake_ctid;
 
 DROP TABLE tid_tab CASCADE;
+
+-- test non-error-throwing API for some core types
+SELECT pg_input_is_valid('(0)', 'tid');
+SELECT pg_input_error_message('(0)', 'tid');
+SELECT pg_input_is_valid('(0,-1)', 'tid');
+SELECT pg_input_error_message('(0,-1)', 'tid');
-- 
2.18.0

